<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Internal library of functions for module releasestatusmw
 *
 * All the releasestatusmw specific functions, needed to implement the module
 * logic, should go here. Never include this file from your lib.php!
 *
 * @package    mod_releasestatusmw
 * @copyright  2016 Your Name <your@email.address>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

/** Include the files that are required by this module */
require_once($CFG->dirroot.'/course/moodleform_mod.php');
require_once($CFG->dirroot . '/mod/releasestatusmw/lib.php');
require_once($CFG->libdir . '/filelib.php');
require_once($CFG->dirroot.'/webservice/rest/lib.php');

/*
 * Does something really useful with the passed things
 *
 * @param array $things
 * @return object
 *function releasestatusmw_do_something_useful(array $things) {
 *    return new stdClass();
 *}
 */

/**
 * Return if the given user has completed his/her releasestatusmw answer.
 *
 * @param  int      $moduleitemid    the releasestatusmw id
 * @param  int      $userid    		 the user id
 * @return boolean  true if has completed or false otherwise.
 * @since  Moodle 3.3
 */
function has_user_answer_completed($moduleitemid, $userid) {
	global $DB, $USER;

	if ($userid === null) {
		$userid = $USER->id;
	}

	$conditions = array('releasestatusmwid' => $moduleitemid, 'userid' => $userid, 'completed' => '1');
	return $DB->count_records('releasestatusmw_answer', $conditions) > 0;	
}

/**
 * Return the answer in the current releasestatusmw for the given user.
 *
 * @param  int      $userid    the user id
 * @param  string   $sort      an order to sort the results in (optional, a valid SQL ORDER BY parameter).
 * @param  string   $fields    a comma separated list of fields to return
 * @param  int      $limitfrom return a subset of records, starting at this point (optional).
 * @param  int      $limitnum  return a subset comprising this many records in total (optional, required if $limitfrom is set).
 * @return array    list of answer for the given user in the releasestatusmw
 * @since  Moodle 3.3
 */
function get_user_releasestatusmw_answer($releasestatusmwid, $userid, $sort = '', $fields = '*', $limitfrom = 0, $limitnum = 0) {
	global $DB, $USER;

	if ($userid === null) {
		$userid = $USER->id;
	}

	$params = array('releasestatusmwid' => $releasestatusmwid, 'userid' => $userid);
	return $DB->get_records('releasestatusmw_answer', $params, $sort, $fields, $limitfrom, $limitnum);
}

/**
 * Start the releasestatusmw answer's of the given user.
 *
 * @param  int      $moduleitemid    the releasestatusmw id
 * @param  int      $userid    		 the user id
 * @param  stdclass      $course    		 the course
 * @return boolean  true if has successfully started or false otherwise.
 * @since  Moodle 3.3
 */
function start_releasestatusmw_answer($releasestatusmwid, $userid, $courseid) {
	global $USER, $DB;

	$answer = get_user_releasestatusmw_answer($releasestatusmwid, $userid, 'starttime DESC', '*', 0, 1);
	if (!$answer) {	
	//print_r($releasestatusmwid);
	//print_r($userid);
	//print_r($courseid);
	$cm = get_coursemodule_from_instance('releasestatusmw', $releasestatusmwid, $courseid,
			false, MUST_EXIST);

	// Trigger releasestatusmw started event.
	$event = \mod_releasestatusmw\event\releasestatusmw_started::create(array(
			'objectid' => $releasestatusmwid,
			'context' => context_module::instance($cm->id),
			'courseid' => $courseid
	));
	$event->trigger();

	$USER->startreleasestatusmw[$releasestatusmwid] = true;

	$timenow = time();
	$startreleasestatusmw = new stdClass;
	$startreleasestatusmw->releasestatusmwid = $releasestatusmwid;
	$startreleasestatusmw->userid = $userid;
	$startreleasestatusmw->starttime = $timenow;
	$startreleasestatusmw->submittime = 0;
	
		$DB->insert_record('releasestatusmw_answer', $startreleasestatusmw);	
	}
	return true;
}

/**
 * Complete the releasestatusmw answer's of the given user.
 *
 * @param  int      $moduleitemid    the releasestatusmw id
 * @param  int      $userid    		 the user id
 * @param  stdclass $courseid    		 the course
 * @return boolean  true if has successfully completed or false otherwise.
 * @since  Moodle 3.3
 */
function end_releasestatusmw_answer($releasestatusmwid, $userid, $courseid) {
	global $USER, $DB;

	$status = false;
	
	$studentCourseStatus = get_student_course_status($userid, $courseid);
	
// 	echo "<br /><br />";
// 	print_r($studentCourseStatus);
// 	echo "<br /><br />";
	
	$studentgrade = $studentCourseStatus->studentgrade;
	$coursemaxgrade = $studentCourseStatus->coursemaxgrade;
	$finishedtime = $studentCourseStatus->finishedtime;
	$passed = verify_student_passed_course($studentgrade, $coursemaxgrade, $finishedtime);
	
	if(!$passed){
		debugging('nota final indisponivel');
		echo "<br />";
		echo get_string("finalgradeunavailable", "releasestatusmw");
		echo "<br /><br />";
	}
	else {			
		// Send student's grade to MentorWeb
		$status = send_grade_to_mw($userid, $courseid, $studentgrade);
		
		// Only update the moodle info if the integration with MentorWeb was successfully executed
		if($status) {	
			unset($USER->startreleasestatusmw[$releasestatusmwid]);
			
			$cm = get_coursemodule_from_instance('releasestatusmw', $releasestatusmwid, $courseid,
					false, MUST_EXIST);
			
			// Trigger releasestatusmw ended event.
			$event = \mod_releasestatusmw\event\releasestatusmw_ended::create(array(
					'objectid' => $releasestatusmwid,
					'context' => context_module::instance($cm->id),
					'courseid' => $courseid
			));
			$event->trigger();
			
			// clock code
			// get time information for this user
			if (!$answer = get_user_releasestatusmw_answer($releasestatusmwid, $userid, 'starttime DESC', '*', 0, 1)) {
				start_releasestatusmw_answer($releasestatusmwid, $userid, $courseid);
				$answer = get_user_releasestatusmw_answer($releasestatusmwid, $userid, 'starttime DESC', '*', 0, 1);
			}
			$answer = current($answer); // This will get the latest start time record.
			
			$timenow = time();
			$answer->submittime = $timenow;	
			$answer->completed = true;
			
			$status = false;
			try {				 
				$transaction = $DB->start_delegated_transaction();
				$status = $DB->update_record('releasestatusmw_answer', $answer);
				$transaction->allow_commit();
				
				// Update completion state.
				$cm = get_coursemodule_from_instance('releasestatusmw', $releasestatusmwid, $courseid,
						false, MUST_EXIST);
				$courseobj = get_course($cm->course);
				$completion = new completion_info($courseobj);
				if ($completion->is_enabled($cm)) {
					$completion->update_state($cm, COMPLETION_COMPLETE);
				}
			} catch(Exception $e) {
				$transaction->rollback($e);
			}			
		}
	}
	
	return $status;	
}


/**
 * Send student's grade to MentorWeb.
 * 
 * @param userid The user id.
 * @param courseid The course id.
 * @param studentgrade The student's grade.
 * 
 * @return status True if grade was successfully sent to MentorWeb, false otherwise.
 */

function send_grade_to_mw($userid, $courseid, $studentgrade) {
	$status = false;
	
	$mwi = new mentorweb_integrator();
	
	// Format the grade as expected 
	$formatedgrade = number_format($studentgrade, 2, '.', '');
	
	$jsonparams = array(mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_STUDENT => $userid, 
						mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_COURSES => 
								array(mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_COURSEID => $courseid,
											mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_GRADE => $formatedgrade
							      )		
				  	   );
	$serviceResult = $mwi->send_grade($jsonparams);	
	
	// 	// TODO: remover
	// 	print_object($serviceResult);
	// 	echo "resultado call service:<br />";
	// 	print_r($serviceResult);
	// 	echo "<br />---<br />";
	
	// Returned JSON as object
	$resultDetailsArray = $serviceResult->getReturnedResult();		 
	
	$status = $serviceResult != NULL && $serviceResult->getCallingStatus() === true && $resultDetailsArray != NULL 
	          && $resultDetailsArray[mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_RESULT_ITEM] === mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_STATUS_SUCCESS;
	
	return $status;
}

/**
 * Verify if the student has finished (completed) and reached the passing score in the course.
 * 
 * @param unknown $studentgrade The student grade in the course.
 * @param unknown $coursemaxgrade The course max grade.
 * @param unknown $finishedtime The time when the student finished the course.
 */
function verify_student_passed_course($studentgrade, $coursemaxgrade, $finishedtime) {	
	// sempre retorna true, pois o acesso a avaliacao do plugin sera restrito a completar outras avaliacoes entao nao valida media
	return true;
}

/**
 * Get the student's course status. 
 * This includes: 
 * 				  the student's grade in the course;
 *                the course's max grade; 
 *                and the time when the student finished (completed) the course.
 * 
 * @param unknown $userid The user id.
 * @param unknown $courseid The course id.
 * 
 * @return stdClass An object to return student's course status.
 */
function get_student_course_status($userid, $courseid) {
	global $DB;

	// Get the student's final grade in the course and the course max grade
	$sql = "SELECT u.id studentid,
			       max(g.finalgrade) studentgrade,
			       gi.grademax coursemaxgrade,
			       COALESCE(cc.timecompleted, 0) finishedtime
	          FROM {user} u
	          JOIN {grade_grades} g ON g.userid = u.id
			  JOIN {grade_items} gi ON g.itemid =  gi.id
	          JOIN {course} c ON c.id = gi.courseid
			  LEFT JOIN {course_completions} cc ON cc.course = c.id AND cc.userid = u.id
             WHERE gi.courseid  = :courseid
			   AND u.id = :userid
			   AND gi.itemtype = 'course'
			group by gi.grademax, u.id ,cc.timecompleted
			ORDER BY 2 DESC";

	$params = array('courseid'=>$courseid, 'userid'=>$userid);

	$rs = $DB->get_record_sql($sql, $params);

	$obj = new stdClass();
	$obj->studentgrade = $rs->studentgrade;
	$obj->coursemaxgrade = $rs->coursemaxgrade;
	$obj->finishedtime = $rs->finishedtime;
	
	return $obj;
}

/**
 * 
 */
class mentorweb_integrator {
	const URL_MENTORWEB = "mwurl";
	//const USERNAME = "usuario"; 
	//const PASSWORD = "senha"; 
	const USERNAME = "mwusername"; //"mentorweb"; 
	const PASSWORD = "mwpassword"; //"123456"; 
	
	const MW_EXT_SERVICE_TOKEN_USER_HEADER = "usuario: ";
	const MW_EXT_SERVICE_TOKEN_PASSWORD_HEADER = "senha: ";
	const MW_EXT_SERVICE_EXECUTE_TOKEN_HEADER = "token: ";
	const MW_EXT_SERVICE_EXECUTE_CONT_TYPE_JSON_HEADER = "Content-Type: application/json";
	
	const MW_EXT_SERVICE_TOKEN_PATH = "mwtokenpath"; //"/rest/servicoexterno/token/";
	const MW_EXT_SERVICE_EXECUTE_PATH = "mwexecpath"; //"/rest/servicoexterno/execute/";
	
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE = "mwsentgradeextservname"; //"integracaoMoodle"; //"gravaMediasFinais";	
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_STUDENT = "aluno";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_COURSES = "disciplinas";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_COURSEID = "idTurmaDisciplina";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_PARAM_GRADE = "nota";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_STATUS_SUCCESS = "SUCESSO";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_STATUS_ERROR = "ERRO";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_RESULT_ITEM = "resultado";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_ERROR_ITEM = "erro";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_ERROR_REASON_ITEM = "motivo";
	const FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_RESULT_DADOS = "dados";
	
	const JSON_FORMAT = "json";
	
	const MW_EXT_SERVICE_MSG_ERR_CONTENT_PREFIX =  "mensagemErro";
		
	const MW_EXT_SERVICE_MSG_ERR_TYPE_UNAVAILABLE = "<br />Servico externo do MentorWeb indisponivel!<br />";	
	const MW_EXT_SERVICE_MSG_ERR_TYPE_TOKENGEN = "<br />Erro ao tentar gerar o token:<br />";
	const MW_EXT_SERVICE_MSG_ERR_TYPE_SERVICE_EXECUTION = "<br /><h2>Erro ao tentar executar o servico externo do MentorWeb:</h2>";
	
	
	//private $moodlewsclient;
	//private $token;
	
	private $releasestatusmwconfig;
	
	function __construct(){
		//$token = $this->generate_token();
		//$moodlewsclient = new webservice_rest_client($serverurl, $token, 'json');
		$this->get_releasestatusmw_config();
	}	
	
	/**
	 *
	 * @param unknown $servicename
	 * @param unknown $jsonparams
	 * @return boolean
	 */
	public function send_grade($jsonparams){
		return $this->servico_externo($this->get_releasestatusmw_config_value(mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE), $jsonparams);
	}
	
	/**
	 * Call the MW external service
	 * 
	 * @deprecated
	 * @param unknown $servico
	 * @param unknown $jsonparams
	 */
	public function servico_externo($servico, $jsonparams){
		return $this->call_service($servico, $jsonparams);
	}
	
	/**
	 * Get the param configured at releasestatusmw plugin's admin settings.
	 *
	 * @param unknown $param
	 */
	private function get_releasestatusmw_config() {
		global $CFG;
	
		// Get the settings object.
		$rsmwconfig = get_config('mod_releasestatusmw');
	
		//// FIXME: Remover
		//print_object($rsmwconfig);
	
		$this->releasestatusmwconfig = $rsmwconfig;
	}
	
	/**
	 * Get the value of the parameter configured at releasestatusmw plugin's admin settings.
	 *
	 * @param unknown $param
	 */
	private function get_releasestatusmw_config_value($param) {
		
		if($this->releasestatusmwconfig == NULL || !isset($this->releasestatusmwconfig->{$param}))
		{
			return NULL;
		}
	
		return $this->releasestatusmwconfig->{$param};
	}
	
	/**
	 * 
	 * @param unknown $servicename
	 */
	private function generate_token($servicename){
		$output = NULL;
		$headers =	array(mentorweb_integrator::MW_EXT_SERVICE_TOKEN_USER_HEADER.
								$this->get_releasestatusmw_config_value(mentorweb_integrator::USERNAME),
						  mentorweb_integrator::MW_EXT_SERVICE_TOKEN_PASSWORD_HEADER.
								$this->get_releasestatusmw_config_value(mentorweb_integrator::PASSWORD));
		
		$output = $this->curl_get($this->get_releasestatusmw_config_value(mentorweb_integrator::URL_MENTORWEB).
								  		$this->get_releasestatusmw_config_value(mentorweb_integrator::MW_EXT_SERVICE_TOKEN_PATH).
				                  		$servicename,
					              $headers
								);
		return $output;	
	}
	
	/**
	 * 
	 * @param unknown $servicename
	 * @param unknown $jsonparams
	 * @return boolean
	 */
	private function call_service($servicename, $jsonparams = NULL){
		debugging('start call_service');		
		$resultDetails = new ServiceResultDetails($servicename, $jsonparams);
		debugging('before generate_token');
		$token = $this->generate_token($servicename);
		if($token == NULL){
			debugging('Null token');
			echo mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_TYPE_UNAVAILABLE;
		}
		else{
			debugging('after generate_token');
			
			$resultDetails->setToken($token);
			
			//print_object($token);
			// mensagemErro
			if(strpos($token, mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_CONTENT_PREFIX) > 0){
				debugging('token has error');
				echo mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_TYPE_TOKENGEN.$token;
				return $resultDetails;
			}		
				
			debugging('token has no error');
						
			debugging('jsonparams');			
				
			$postResult = $this->curl_post($this->get_releasestatusmw_config_value(mentorweb_integrator::URL_MENTORWEB).
												$this->get_releasestatusmw_config_value(mentorweb_integrator::MW_EXT_SERVICE_EXECUTE_PATH).
												$servicename,
										   $jsonparams,
											array(mentorweb_integrator::MW_EXT_SERVICE_EXECUTE_TOKEN_HEADER.
														$token,
												  mentorweb_integrator::MW_EXT_SERVICE_EXECUTE_CONT_TYPE_JSON_HEADER											
											)
										);
			
			debugging('curl_post success');
			//print_object($result);
			
			$resultDetails->setResult($postResult);
						
			if($servicename != $this->get_releasestatusmw_config_value(mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE)){
				// Case of sgdd integration
				if(strpos(implode(" ", $postResult), mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_CONTENT_PREFIX) <= 0){
					$resultDetails->setSuccess();
					
					print_object($postResult);
				}
				else{
					debugging('result has error');
					echo mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_TYPE_SERVICE_EXECUTION.$postResult;
				}
			}
			else {
				// Case of final grade to mw
				$returnedResult = $postResult[mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_RESULT_ITEM];

				$dados = $postResult[mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_RESULT_DADOS];
				
				if($dados != null && !empty($dados)){
					$msgLog = "<b>Veja abaixo sua(s) pendencia(s), e entre em contato com a instituição para regularizar a sua situação:</b>";
					$msgLog .= "<br /><br />"; 
					
					echo "<ul>";
					foreach ($dados as $resultDados) {
					    $msgLog .= "<li>".$resultDados."</li>";
					}
					echo "</ul>";
					
					echo $msgLog;
				}

				if($returnedResult === mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_STATUS_SUCCESS){
					$resultDetails->setSuccess();
				}
				else{				
					debugging('result has error');
					$returnedErrorDetails = $postResult[mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_ERROR_ITEM];
					$errorReason = $returnedErrorDetails[mentorweb_integrator::FINAL_GRADE_MDL_TO_MW_EXT_SERVICE_ERROR_REASON_ITEM];
					if($errorReason == NULL || !$errorReason){
						$errorReason = mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_TYPE_UNAVAILABLE;
					}
					
					$errorReason = str_replace("EduClientLogProcessoException:", "", $errorReason);
					
					echo mentorweb_integrator::MW_EXT_SERVICE_MSG_ERR_TYPE_SERVICE_EXECUTION."<h3><br /><b>Atenção:</b>".$errorReason."</h3>";
				}
			}			
			
			debugging('end call_service');
		}
			
		return $resultDetails;
	}
	
	/**
	 * Send a GET requst using cURL
	 * @param string $url to request
	 * @param array $get values to send
	 * @param array $options for cURL
	 * @return string
	 */
	private function curl_get($url, $headers = NULL, array $get = NULL)
	{
		debugging('curl_get');
		
		// Init the curl resource
		$ch = curl_init();
		
		debugging('get curl_init');
	
		$urlwithparams = $url. (strpos($url, '?') === false ? '?' : ''). ($get == NULL ? '' : http_build_query($get));
		
		debugging('get URL params');
	
		// Define the curl options' values
		curl_setopt($ch, CURLOPT_URL, $urlwithparams);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 100);
		
		debugging('get curl_setopt');
	
		// If there are headers values, so configure them
		if($headers != NULL) {
			//curl_setopt($ch, CURLOPT_HEADEROPT, CURLHEADER_UNIFIED);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}
	
		debugging('get headers');
		
		// Try to execute the curl request
		if( ! $result = curl_exec($ch))
		{
			debugging('get curl_exec error');
			// If some error occurs, trigger it
			trigger_error(curl_error($ch));
		}
		
		debugging('get curl_exec success');
		//print_object($result);
	
		// Close the curl resource
		curl_close($ch);
		
		debugging('get curl_close');
	
		return $result;
	}
			
	/**
	 * Send a POST requst using cURL
	 * @param string $url to request
	 * @param array $post values to send
	 * @param array $options for cURL
	 * @return string
	 */
	function curl_post($url, array $post = NULL, $headers = NULL, $format = mentorweb_integrator::JSON_FORMAT)
	{
		debugging('curl_post');
		
		// Init the curl resource
		$ch = curl_init();
		
		debugging('post curl_init');
	
		// Define the curl options' values
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 100);
	
		debugging('post curl_setopt');
		
		// Add the post fields to the curl resource
		if($format == mentorweb_integrator::JSON_FORMAT){			
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
			debugging('post json');
		}
		else {
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
			debugging('post http query');
		}
	
		// If there are headers values, so configure them
		if($headers != NULL) {
			//curl_setopt($ch, CURLOPT_HEADEROPT, CURLHEADER_UNIFIED);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			debugging('post headers');
		}
	
		// Try to execute the curl request
		if( ! $result = curl_exec($ch))
		{
			debugging('post curl_exec error');
			// If some error occurs, trigger it
			trigger_error(curl_error($ch));
		}
	
		debugging('post curl_exec success');
		//print_object($result);
		
		// Close the curl resource
		curl_close($ch);
		
		debugging('post curl_close');
	
		// Decode the JSON result, just in the case
		if($format == mentorweb_integrator::JSON_FORMAT){
			$result = json_decode($result, true);
		}
		
		debugging('post json decode');
		//print_object($result);
	
		return $result;
	}
}

class ServiceResultDetails {

	private $calledService;
	private $passedParams;
	private $usedToken;
	
	private $callingStatus;
	private $returnedResult;

	public function __construct($service, $params){
		$this->calledService = $service;
		$this->passedParams = $params;
		$this->usedToken = NULL;
		$this->callingStatus = false;
		$this->returnedResult = NULL;
		//$moodlewsclient = new webservice_rest_client($serverurl, $token, 'json');
	}
	
	public function setToken($token){
		$this->usedToken = $token;
	}
	
	public function setSuccess(){
		$this->callingStatus = true;
	}
	
	public function setResult($result){
		$this->returnedResult = $result;
	}
	
	public function getServiceName(){
		return $this->calledService;
	}
	
	public function getCallingStatus(){
		return $this->callingStatus;
	}
	
	public function getReturnedResult(){
		return $this->returnedResult;
	}	
	
	public function __toString(){
		return "ServiceResultDetails [calledService:".$this->calledService.",
				                      passedParams:".$this->passedParams.", 
				                      usedToken:".$this->usedToken.", 
				                      callingStatus:".$this->callingStatus.",
				                      returnedResult:".$this->returnedResult."]";
	}
	
}